"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const execa = require("execa");
const path = require("path");
const fs = require("fs");
const mkdirp = require("mkdirp");
const Command_1 = require("../Command");
const util_1 = require("../util");
class Setup extends Command_1.Command {
    constructor(args) {
        super(args);
    }
    cloneTypeDefinitions() {
        util_1.logger.warn("setting up typeDefs, this could take a while... and a lot of bandwith...");
        for (let [typeDefName, typeInfo] of Object.entries(this.config.typeDefs)) {
            const typeDefPath = path.join(this.config.baseDir, "@types", typeDefName);
            if (!fs.existsSync(typeDefPath)) {
                mkdirp.sync(typeDefPath);
            }
            if (fs.existsSync(path.join(typeDefPath, ".git"))) {
                return;
            }
            execa.sync("git", ["clone", typeInfo.remote, typeDefPath, "--branch", typeInfo.branch, "--depth", "1"], {
                cwd: typeDefPath
            });
            return util_1.logger.info(`cloned typeDefinition for ${typeDefName}`);
        }
    }
    setupWorkingDirectory() {
        const cwd = path.join(this.config.baseDir);
        let isGitRepo = true;
        try {
            execa.sync("git", ["status"], { cwd });
        }
        catch (err) {
            isGitRepo = false;
        }
        if (isGitRepo) {
            util_1.logger.error(`cannot intialize in a git repository`);
            process.exit(1);
            return;
        }
        if (!fs.existsSync(cwd)) {
            util_1.logger.warn(`creating working directory`);
            mkdirp.sync(this.config.baseDir);
        }
    }
    cloneAndInstall(projectDir, project, countOf) {
        super.cloneProject(project.remote, projectDir);
        super.checkoutBranch(projectDir, project.branch);
        util_1.logger.iterateInfo(`cloned and checked out ${project.package}#${project.branch}`, countOf);
        super.installDependencies(projectDir, project, countOf);
    }
    setupProject(projectDir, project, countOf) {
        if (project.types) {
            super.linkTypes(projectDir, project);
        }
        super.linkDependencies(projectDir, project, countOf);
        super.buildProject(projectDir, project, countOf);
        super.linkSelf(projectDir, project);
    }
    async run() {
        this.setupWorkingDirectory();
        await Promise.all([this.cloneTypeDefinitions(), super.eachProject(this.cloneAndInstall)]);
        await super.eachProject(this.setupProject.bind(this));
    }
}
Setup.commandName = "setup";
Setup.commandHelp = `
sets up project workspace, 
clones and installs projects, type definitions, 
builds and links dependencies

Usage:
  $ yarn-compose 
    (expects projects.yml)
  $ yarn-compose -c path/to/config.yml
  `;
exports.Setup = Setup;
