"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const meow = require("meow");
const util_1 = require("./util");
const Setup_1 = require("./commands/Setup");
const Relink_1 = require("./commands/Relink");
const Rebuild_1 = require("./commands/Rebuild");
const Command_1 = require("./Command");
const defaultFlags = {
    buildDir: {
        name: "buildDir",
        alias: "t"
    },
    configPath: {
        name: "configPath",
        alias: "c"
    }
};
const meowConfig = {
    flags: defaultFlags,
    autoHelp: false,
    argv: process.argv.slice(3)
};
const defaultHelpOptions = `
Global Options:

--base-dir, -t
  specify which build directory to target.
  overrides config file value

--config-path, -c
  specify path to config file
  ./projects.yml by default
`;
const commands = {
    setup: Setup_1.Setup,
    relink: Relink_1.Relink,
    rebuild: Rebuild_1.Rebuild,
    none: Command_1.Command
};
(async () => {
    try {
        const [, , commandArg] = process.argv;
        const SelectedCmd = commandArg ? commands[commandArg] || commands.none : commands.none;
        util_1.logger.meta(SelectedCmd.commandName);
        if (SelectedCmd.additionalFlags) {
            meowConfig.flags = Object.assign(meowConfig.flags, SelectedCmd.additionalFlags);
        }
        const otherArgs = meow(SelectedCmd.commandHelp, meowConfig);
        if (otherArgs.flags.help) {
            util_1.logger.help(SelectedCmd.commandHelp + "\n" + defaultHelpOptions);
            process.exit(0);
        }
        const Cmd = new SelectedCmd(otherArgs);
        await Cmd.run();
    }
    catch (err) {
        util_1.logger.error(err);
    }
})();
